#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("DoDeletionBeforeCoveringOperator", "Description missing")]
  [StorableClass]
  public class DoDeletionBeforeCoveringOperator : SingleSuccessorOperator {

    #region Parameter Properties
    public ILookupParameter<ItemArray<IClassifier>> ClassifiersParameter {
      get { return (ILookupParameter<ItemArray<IClassifier>>)Parameters["Classifiers"]; }
    }
    public ILookupParameter<ItemArray<BoolValue>> MatchConditionParameter {
      get { return (ILookupParameter<ItemArray<BoolValue>>)Parameters["MatchCondition"]; }
    }
    public ILookupParameter<IntValue> MinimalNumberOfUniqueActionsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["MinimalNumberOfUniqueActions"]; }
    }
    public ILookupParameter<IntValue> CurrentPopulationSizeParameter {
      get { return (ILookupParameter<IntValue>)Parameters["CurrentPopulationSize"]; }
    }
    public ILookupParameter<IntValue> PopulationSizeParameter {
      get { return (ILookupParameter<IntValue>)Parameters["PopulationSize"]; }
    }
    public IValueLookupParameter<BoolValue> DoDeletionParameter {
      get { return (IValueLookupParameter<BoolValue>)Parameters["DoDeletion"]; }
    }
    public IValueLookupParameter<IntValue> NumberToDeleteParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["NumberToDelete"]; }
    }
    public ILookupParameter<IClassifierComparer> ClassifierComparerParameter {
      get { return (ILookupParameter<IClassifierComparer>)Parameters["ClassifierComparer"]; }
    }
    #endregion

    [StorableConstructor]
    protected DoDeletionBeforeCoveringOperator(bool deserializing) : base(deserializing) { }
    protected DoDeletionBeforeCoveringOperator(DoDeletionBeforeCoveringOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public DoDeletionBeforeCoveringOperator()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<IClassifier>("Classifiers"));
      Parameters.Add(new ScopeTreeLookupParameter<BoolValue>("MatchCondition"));
      Parameters.Add(new LookupParameter<IntValue>("MinimalNumberOfUniqueActions"));
      Parameters.Add(new LookupParameter<IntValue>("CurrentPopulationSize"));
      Parameters.Add(new LookupParameter<IntValue>("PopulationSize"));
      Parameters.Add(new ValueLookupParameter<BoolValue>("DoDeletion"));
      Parameters.Add(new ValueLookupParameter<IntValue>("NumberToDelete"));
      Parameters.Add(new LookupParameter<IClassifierComparer>("ClassifierComparer"));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DoDeletionBeforeCoveringOperator(this, cloner);
    }

    public override IOperation Apply() {
      IItemSet<IAction> uniqueActions = new ItemSet<IAction>(ClassifierComparerParameter.ActualValue);
      var classifiers = ClassifiersParameter.ActualValue;
      var matchcondition = MatchConditionParameter.ActualValue;

      if (classifiers.Length != matchcondition.Length) {
        throw new ArgumentException("Number of classifiers is not equal to the number of 'MatchCondition' variables.");
      }

      for (int i = 0; i < classifiers.Length; i++) {
        if (matchcondition[i].Value) {
          uniqueActions.Add(classifiers[i].Action);
        }
      }

      int populationAfterCovering = MinimalNumberOfUniqueActionsParameter.ActualValue.Value
                                    - uniqueActions.Count
                                    + CurrentPopulationSizeParameter.ActualValue.Value;
      BoolValue doDeletion = new BoolValue(populationAfterCovering > PopulationSizeParameter.ActualValue.Value);
      if (doDeletion.Value) {
        NumberToDeleteParameter.ActualValue = new IntValue(populationAfterCovering - PopulationSizeParameter.ActualValue.Value);
      } else {
        NumberToDeleteParameter.ActualValue = new IntValue(0);
      }
      DoDeletionParameter.ActualValue = doDeletion;
      return base.Apply();
    }
  }
}